DATA_SECTION
	init_int nobs								//Number of observations
	init_int nf_nb								//Number of Fixed effects parameters for negative binomial
	init_int nf_p								//Number of Fixed effects parameters for zero inflation
	init_matrix fdesign_nb(1,nobs,1,nf_nb) 		//Design matrix from R containing intercept & fixed effects
	init_matrix fdesign_p(1,nobs,1,nf_p) 		//Design matrix from R containing intercept & fixed effects
	init_ivector obs(1,nobs)					//Observations of the response variable
	
	init_int eofc;
	!! if(eofc!=999){cout<<"Error in control file.\n"; exit(1);}

PARAMETER_SECTION
	init_vector fcoeffs_nb(1,nf_nb) 				//Coefficients for the fixed effects and intercept (Negative Binomial part)
  !! fcoeffs_nb.set_scalefactor(.1);
	init_vector fcoeffs_p(1,nf_p) 				//Coefficients for the fixed effects and intercept (Zero Inflation part)
  !! fcoeffs_nb.set_scalefactor(.001);
	init_bounded_number k(1.01,20)				//Overdispersion Parameter for Negative Binomial Distribution
  !!k.set_scalefactor(.1);
	vector eta(1,nobs)
	vector mu(1,nobs)
	vector logitp(1,nobs)
	vector p(1,nobs)

	objective_function_value nll

PROCEDURE_SECTION
	nll=0.0;
	dvariable fpen=0.0;

	eta=fdesign_nb*fcoeffs_nb;
	mu=mfexp(eta);
	
	logitp=fdesign_p*fcoeffs_p;

	p=1.0/(1.0+mfexp(-logitp));	//logistic function
	
	// penalties: constrain 0.001 <= prob <= 0.999
	p = posfun(p,0.001,fpen);
	nll += 1.0e16*fpen;
	p = 1-posfun(1-p,0.001,fpen);
	nll += 1.0e16*fpen;
	if(fpen>0){cout<<"fpen"<<fpen<<endl<<endl;}
	for(int i=1; i<=nobs; i++)
	{
		if(obs(i)==0)
		{
			nll-=log(p(i) + (1-p(i))* pow(k/(k+mu(i)),k) );
		}
		else//obs(i)>0
		{
			nll-=log(1-p(i)) + gammln(k+obs(i))-gammln(k)-gammln(obs(i)+1)+k*log(k)-k*log(mu(i)+k)+obs(i)*log(mu(i))-obs(i)*log(mu(i)+k);
		}
	}

TOP_OF_MAIN_SECTION
	arrmblsize = 200000;
	gradient_structure::set_GRADSTACK_BUFFER_SIZE(1000);
	//gradient_structure::set_CMPDIF_BUFFER_SIZE(41800000);
	//gradient_structure::set_MAX_NVAR_OFFSET(5000);
	//gradient_structure::set_NUM_DEPENDENT_VARIABLES(5000);
	
	
	

